#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <ctype.h>
#include <string.h>
#include <time.h>
#include "xmalloc.h"
#include "laroec.h"

static long *phrase_pos;
static char *phrase_data;
static long nphrases;

long 
count_phrases(char *p) 
{
        long count;
        for(count = 0; (p= index(p, CHAR_ENDPHRASE)) != NULL; count++,p++);
        return(count);
}

long *
fill_phrase_positions(char *p, long phrases)
{
        long *pos;
        int i;
        char *s = p;
        pos = xmalloc(sizeof(long) * (phrases + 1));
        for(i=0; (i < phrases) && ((s = index(s, CHAR_ENDPHRASE)) != NULL); i++) 
                pos[i] = s++ - p;
        return(pos);
}

char *
open_and_map(char *filename)
{
     struct stat st;
     int fd;

     if ( stat(filename, &st) < 0 ) {
	  perror("laroec");
	  return(NULL);
     }
     fd = open(filename, O_RDONLY);
     return((char *) mmap(0, st.st_size, PROT_READ, MAP_SHARED, fd, 0));
}


char *
get_random_phrase() 
{
     static char result[1024];
     char *p, *tmp;
     p = phrase_data + phrase_pos[(long) (((double) nphrases) * (rand()/(double) RAND_MAX))];
     p++;
     while ( isspace(*p) ) p++;
     tmp = p;
     while ( *tmp != CHAR_ENDPHRASE && *tmp ) 
	  tmp++;
     strncpy(result, p, tmp - p);
     result[tmp - p] = '\0';
     return(result);
}

int
initialize_laroec(int type)
{
     char *file;
     switch(type) {
     case TYPE_IFTHEN:
	  file = DEFAULT_IFTHEN_LIB;
	  break;
     case TYPE_COMESFROM:
	  file = DEFAULT_COMESFROM_LIB;
	  break;
     }
     phrase_data = open_and_map(file);
     nphrases = count_phrases(phrase_data);
     phrase_pos = fill_phrase_positions(phrase_data, nphrases);
     srand(time(NULL));
     return(1);
}
	  
int
main(int argc, char **argv)
{
#ifdef CURSES
     curses_ui_main(argc, argv);
#else
     text_ui_main(argc, argv); 
#endif
     return(1);
}

